use anyhow::{Context, Result};
use poise::serenity_prelude::{EmbedMessageBuilding, MessageBuilder};
use songbird::create_player;
use tracing::debug;

use crate::{CommandContext, Error};

#[poise::command(slash_command)]
pub async fn join(ctx: CommandContext<'_>) -> Result<(), Error> {
    let Some(guild) = ctx.guild() else {
        ctx.say("You're not in a server, silly.").await?;
        return Ok(());
    };

    let Some(Some(channel_id)) = guild.voice_states.get(&ctx.author().id).map(|vs| vs.channel_id) else {
        ctx.say("You're not in a voice channel, silly.").await?;
        return Ok(());
    };

    let manager = songbird::get(ctx.serenity_context())
        .await
        .context("Expected a songbird manager")?
        .clone();
    let _handler = manager.join(guild.id, channel_id).await;

    ctx.say("Joining your channel!").await?;

    Ok(())
}

#[poise::command(slash_command)]
pub async fn leave(ctx: CommandContext<'_>) -> Result<(), Error> {
    let Some(guild) = ctx.guild() else {
        ctx.say("You're not in a server, silly.").await?;
        return Ok(());
    };

    let manager = songbird::get(ctx.serenity_context())
        .await
        .context("Expected a songbird manager")?
        .clone();

    if manager.get(guild.id).is_none() {
        ctx.say("I'm not even in a voice channel!").await?;
        return Ok(());
    }

    let _handler = manager.remove(guild.id).await;

    ctx.say("Okay bye!").await?;

    Ok(())
}

#[poise::command(slash_command)]
pub async fn play(
    ctx: CommandContext<'_>,
    #[description = "The URL of the song to play"] url: Option<String>,
) -> Result<(), Error> {
    let Some(url) = url else {
        ctx.say("You need to give me a URL to play!").await?;
        return Ok(());
    };

    let Some(guild) = ctx.guild() else {
        ctx.say("You're not in a server, silly.").await?;
        return Ok(());
    };

    let manager = songbird::get(ctx.serenity_context())
        .await
        .context("Expected a songbird manager")?
        .clone();

    if manager.get(guild.id).is_none() {
        let Some(Some(channel_id)) = guild.voice_states.get(&ctx.author().id).map(|vs| vs.channel_id) else {
            ctx.say("Neither of us are in a voice channel, silly.").await?;
            return Ok(());
        };
        let _handler = manager.join(guild.id, channel_id).await;
    }

    if let Some(handler_lock) = manager.get(guild.id) {
        let mut handler = handler_lock.lock().await;

        debug!("Trying to play: {}", url);
        let source = songbird::ytdl(&url).await?;
        debug!("Playing: {:?}", source.metadata);
        let title = source
            .metadata
            .title
            .clone()
            .unwrap_or(String::from("This video"));
        let msg = MessageBuilder::new()
            .push("Now playing: ")
            .push_named_link(title, url)
            .build();
        ctx.say(msg).await?;

        let (audio, track_handle) = create_player(source);
        let mut currently_playing = ctx.data().currently_playing.lock();
        *currently_playing = Some(track_handle);
        handler.play_only(audio);
    } else {
        ctx.say("Neither of us are in a voice channel, silly.")
            .await?;
    }

    Ok(())
}
