#![warn(clippy::all)]

mod commands;

use commands::*;

use std::{env, sync::Arc};

use anyhow::Result;
use parking_lot::Mutex;
use poise::serenity_prelude::{self as serenity};
use songbird::serenity::SerenityInit;
use tracing_subscriber::filter;

pub struct Data {
    currently_playing: Arc<Mutex<Option<songbird::tracks::TrackHandle>>>,
}
pub type Error = Box<dyn std::error::Error + Send + Sync>;
pub type CommandContext<'a> = poise::Context<'a, Data, Error>;

async fn event_event_handler(
    _ctx: &serenity::Context,
    event: &poise::Event<'_>,
    _framework: poise::FrameworkContext<'_, Data, Error>,
    _user_data: &Data,
) -> Result<(), Error> {
    if let poise::Event::Ready { data_about_bot } = event {
        println!("{} is connected!", data_about_bot.user.name)
    }

    Ok(())
}

/// Registers slash commands in this guild or globally
#[poise::command(prefix_command, hide_in_help)]
async fn register(ctx: CommandContext<'_>) -> Result<(), Error> {
    poise::builtins::register_application_commands_buttons(ctx).await?;

    Ok(())
}

#[tokio::main]
async fn main() -> Result<()> {
    tracing_subscriber::fmt()
        .with_max_level(filter::LevelFilter::DEBUG)
        .init();

    let token =
        env::var("DISCORD_TOKEN").expect("Expected a bot token in the environment: DISCORD_TOKEN");

    let options = poise::FrameworkOptions {
        commands: vec![register(), join(), leave(), play()],
        event_handler: |ctx, event, framework, user_data| {
            Box::pin(event_event_handler(ctx, event, framework, user_data))
        },
        prefix_options: poise::PrefixFrameworkOptions {
            prefix: Some(String::from("~")),
            ..Default::default()
        },
        ..Default::default()
    };

    let intents = serenity::GatewayIntents::non_privileged();

    poise::Framework::builder()
        .token(token)
        .options(options)
        .intents(intents)
        .setup(|_ctx, _data, _framework| {
            Box::pin(async move {
                Ok(Data {
                    currently_playing: Arc::new(Mutex::new(None)),
                })
            })
        })
        .client_settings(|client_builder| client_builder.register_songbird())
        .run()
        .await?;

    Ok(())
}
